/*
 * feedbackd
 *
 * Copyright (c) 2012 - 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <errno.h>

#include "core/log.h"
#include "core/list.h"
#include "haptic.h"

static dd_list *handle_list;
static int unique_number = 0;

/* START: Haptic Module APIs */
static int get_device_count(int *count)
{
	if (count)
		*count = 1;

	return 0;
}

static int open_device(int device_index, int *device_handle)
{
	dd_list *elem;
	int handle;
	bool found = false;

	if (unique_number == INT_MAX)
		unique_number = 0;

	while (found != true) {
		++unique_number;
		elem = DD_LIST_FIND(handle_list, (gpointer)(long)unique_number);
		if (!elem)
			found = true;
	}
	handle = unique_number;
	DD_LIST_APPEND(handle_list, (gpointer)(long)handle);

	if (device_handle)
		*device_handle = handle;

	return 0;
}

static int close_device(int device_handle)
{
	DD_LIST_REMOVE(handle_list, (gpointer)(long)device_handle);

	return 0;
}

static int vibrate_monotone(int device_handle, int duration, int feedback, int priority)
{
	dd_list *elem;

	elem = DD_LIST_FIND(handle_list, (gpointer)(long)device_handle);
	if (!elem)
		return -EINVAL;

	return 0;
}

static int stop_device(int device_handle)
{
	dd_list *elem;

	elem = DD_LIST_FIND(handle_list, (gpointer)(long)device_handle);
	if (!elem)
		return -EINVAL;

	return 0;
}
/* END: Haptic Module APIs */

static const struct haptic_plugin_ops default_plugin = {
	.get_device_count    = get_device_count,
	.open_device         = open_device,
	.close_device        = close_device,
	.vibrate_monotone    = vibrate_monotone,
	.stop_device         = stop_device,
};

static bool is_valid(void)
{
	if (is_emulator()) {
		_I("Support emulator haptic device.");
		return true;
	}

	_E("Do not support emulator haptic device.");
	return false;
}

static const struct haptic_plugin_ops *load(void)
{
	return &default_plugin;
}

static const struct haptic_ops emul_ops = {
	.is_valid = is_valid,
	.load     = load,
};

HAPTIC_OPS_REGISTER(&emul_ops)
